@@ -23,7 +23,7 @@
 #' # to fix the starting point
 #' lowpass(c(0,9,1,5,14,20,6,11,0), i=2, fix="start")
 #' 
-#' @export lowpass
+#' @export
 lowpass <- function(x,i=1, fix=NULL, altFilter=NULL, warn=TRUE) {
   
   if(warn && !is.null(fix)) warning("Fixing start or end does modify the total sum of values! Use fix=NULL to let the total sum unchanged!")
@@ -31,17 +31,34 @@ lowpass <- function(x,i=1, fix=NULL, altFilter=NULL, warn=TRUE) {
   if(i==0) return(x)
   
   if(is.magpie(x)) {
-    for(k in 1:dim(x)[1]) {
-      for(j in if(is.null(getNames(x))) 1 else getNames(x)) {
-        x[k,,j] <- lowpass(as.vector(x[k,,j]),i=i,fix=fix,altFilter=altFilter)
+    years <- getYears(x,as.integer=TRUE)
+    if(is.unsorted(years)) {
+      x <- x[,sort(years),]
+    }
+    l <- nyears(x)
+    x2 <- x
+    x <- as.array(x)
+    for(j in 1:i) {
+      y <- x
+      x[,2:(l-1),] <- (y[,1:(l-2),] + 2*y[,2:(l-1),] + y[,3:l,])/4
+      if(!is.null(altFilter))
+        x[,altFilter,] <- (2*y[,altFilter,] + y[,altFilter+1,])/3  
+      if(is.null(fix)){
+        x[,1,] <- (3*y[,1,]+y[,2,])/4
+        x[,l,] <- (3*y[,l,]+y[,l-1,])/4
       }
-    }  
+      else if (fix=="start") x[,l,] <- (3*y[,l,]+y[,l-1,])/4
+      else if (fix=="end")   x[,1,] <- (3*y[,1,]+y[,2,])/4
+      else if (fix!="both") stop(paste("Option \"",fix,"\" is not available for the \"fix\" argunemt!",sep=""))    
+    }
+    x <- as.magpie(x)
+    copy.attributes(x) <- x2
   } else {
     l <- length(x)
     for(j in 1:i) {
       y <- x
       x[2:(l-1)] <- (y[1:(l-2)] + 2*y[2:(l-1)] + y[3:l])/4
-	  if(!is.null(altFilter))
+      if(!is.null(altFilter))
         x[altFilter] <- (2*y[altFilter] + y[altFilter+1])/3  
       if(is.null(fix)){
         x[1] <- (3*y[1]+y[2])/4
@@ -53,3 +70,4 @@ lowpass <- function(x,i=1, fix=NULL, altFilter=NULL, warn=TRUE) {
   }  
   return(x)
 }
+
